@@ -75,7 +75,7 @@ foreach my $line (<DATA>) {
     print $OUT $line;
 }
 foreach my $theme ( keys %{ $table->get_available_themes } ) {
-    print $OUT "\\section{$theme}\n \\input{$theme.tex}\n";
+    print $OUT "\\subsection{$theme Theme}\n \\input{$theme.tex}\n";
     print $OUT "\\input{${theme}multipage.tex} \\clearpage \\newpage\n";
 }
 
@@ -101,7 +101,7 @@ $table = LaTeX::Table->new(
     }
 );
 
-print ${OUT} '\section{Text::Wrap example}' . "\n";
+print ${OUT} '\section{Advanced Features}\subsection{Text wrapping}' . "\n";
 print ${OUT} <<'EOT'
 Table~\ref{wrap1} demonstrates the \texttt{text\_wrap} feature. 
 EOT
@@ -124,16 +124,20 @@ print ${OUT} "\\end{document}\n";
 close $OUT;
 
 __DATA__
-\documentclass[twoside,12pt]{report}
-\usepackage[margin=10pt,font=small,labelfont=bf]{caption}
+\documentclass{article}
 \usepackage{url}
 \usepackage{graphics, graphicx}
 \usepackage{xtab}
 \begin{document}
-\chapter{Examples}
+\title{LaTeX::Table}
+\date{\today}
+\author{Markus Riester}
+\maketitle
+\section{About this document}
 This document is generated by \texttt{generate\_examples.pl} from
 \textsc{LaTeX::Table}. For each theme, two example tables are shown, one of type
 \textit{std}, the standard \LaTeX~table, and one of type \textit{xtab}. Note that the
 second column is left justified because this column contains numbers and
 strings, whereas the third column is a number column and therefore right
 justified.
+\section{Themes}
